package lt.marsrutai.mobile.serialize;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import lt.marsrutai.mobile.serialize.KryoSerializerMap.SerializerData;
import lt.marsrutai.serialize.exception.SerializeException;

import com.esotericsoftware.kryo.ObjectBuffer;

public class KryoSerializer implements Serializer {

    private static final int BUFFER_SIZE = 1024 * 256;

    @Override
    public byte[] writeObject(SerializeRequest request) {

        SerializerData serializerData = KryoSerializerMap.get(request.getName());

        ObjectBuffer buffer = new ObjectBuffer(serializerData.getKryo(), BUFFER_SIZE);

        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            GZIPOutputStream gos = new GZIPOutputStream(bos);
            buffer.writeObject(gos, request.getObject());
            gos.flush();
            gos.close();

            return bos.toByteArray();
        }
        catch (IOException e) {
            throw new SerializeException(e);
        }
    }

    @Override
    public Object readObject(DeserializeRequest request) {

        SerializerData serializerData = KryoSerializerMap.get(request.getName());

        ObjectBuffer buffer = new ObjectBuffer(serializerData.getKryo(), BUFFER_SIZE);

        try {
            GZIPInputStream gis = new GZIPInputStream(new ByteArrayInputStream(request.getArray()));
            return buffer.readObject(gis, serializerData.getCls());
        }
        catch (IOException e) {
            throw new SerializeException(e);
        }
    }

}
