package lt.marsrutai.mobile.serialize;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import lt.marsrutai.mobile.android.dao.dto.BusStopDto;
import lt.marsrutai.mobile.android.dao.dto.ScheduleDto;
import lt.marsrutai.mobile.android.dao.dto.ScopeInfoDto;
import lt.marsrutai.mobile.android.dao.dto.TransportDto;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryo.Serializer;
import com.esotericsoftware.kryo.serialize.CollectionSerializer;
import com.esotericsoftware.kryo.serialize.FieldSerializer;
import com.esotericsoftware.kryo.serialize.FieldSerializer.CachedField;

public class KryoSerializerMap {

    private static Map<String, SerializerData> map = new HashMap<String, SerializerData>();

    static {
        map.put("getAvailableMobileScopes", getAvailableMobileScopes());
        map.put("getSchedules", getSchedules());
        map.put("getStops", getStops());
    }

    private static SerializerData getAvailableMobileScopes() {
        Kryo kryo = new Kryo();
        kryo.register(ArrayList.class);
        kryo.register(ScopeInfoDto.class);
        kryo.register(TransportDto.class);

        CollectionSerializer serializer = (CollectionSerializer) kryo.getSerializer(ArrayList.class);
        serializer.setElementClass(ScopeInfoDto.class);
        serializer.setElementsCanBeNull(false);

        FieldSerializer fieldSerializer = (FieldSerializer) kryo.getSerializer(ScopeInfoDto.class);
        CachedField transportsField = fieldSerializer.getField("transports");
        CollectionSerializer collectionSerilizer = new CollectionSerializer(kryo);
        collectionSerilizer.setElementClass(TransportDto.class);
        collectionSerilizer.setElementsCanBeNull(false);
        transportsField.setClass(ArrayList.class, collectionSerilizer);

        return new SerializerData(kryo, serializer, ArrayList.class);
    }

    private static SerializerData getSchedules() {
        Kryo kryo = new Kryo();
        kryo.register(ArrayList.class);
        kryo.register(ScheduleDto.class);

        CollectionSerializer serializer = (CollectionSerializer) kryo.getSerializer(ArrayList.class);
        serializer.setElementClass(ScheduleDto.class);
        serializer.setElementsCanBeNull(false);

        return new SerializerData(kryo, serializer, ArrayList.class);
    }

    private static SerializerData getStops() {
        Kryo kryo = new Kryo();
        kryo.register(ArrayList.class);
        kryo.register(BusStopDto.class);

        CollectionSerializer serializer = (CollectionSerializer) kryo.getSerializer(ArrayList.class);
        serializer.setElementClass(BusStopDto.class);
        serializer.setElementsCanBeNull(false);

        return new SerializerData(kryo, serializer, ArrayList.class);
    }

    public static SerializerData get(String key) {
        if (!map.containsKey(key)) {
            throw new IllegalArgumentException("No such key in SerializerMap: " + key);
        }
        return map.get(key);
    }

    public static class SerializerData {

        private Kryo kryo;
        private Serializer serializer;
        private Class<?> cls;

        public SerializerData(Kryo kryo, Serializer serializer, Class<?> cls) {
            this.kryo = kryo;
            this.serializer = serializer;
            this.cls = cls;
        }

        public Kryo getKryo() {
            return kryo;
        }

        public void setKryo(Kryo kryo) {
            this.kryo = kryo;
        }

        public Serializer getSerializer() {
            return serializer;
        }

        public void setSerializer(Serializer serializer) {
            this.serializer = serializer;
        }

        public Class<?> getCls() {
            return cls;
        }

        public void setCls(Class<?> cls) {
            this.cls = cls;
        }

    }
}
